// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import 'package:netease_common/netease_common.dart';
import 'package:netease_roomkit_interface/netease_roomkit_interface.dart';
import 'package:netease_roomkit_interface/src/pigeon.dart';

class MethodChannelSeatEventSink extends SeatEventSink {
  final _listeners = <String, Set<SeatEventSink>>{};
  String _roomUuid = "";
  void registerEventListener(String roomUuid, SeatEventSink listener) {
    _roomUuid = roomUuid;
    _listeners.putIfAbsent(roomUuid, () => <SeatEventSink>{}).add(listener);
  }

  void unregisterEventListener(String roomUuid, SeatEventSink listener) {
    _listeners[roomUuid]?.remove(listener);
  }

  @override
  void onSeatInvitationAccepted(int seatIndex, String user, bool isAutoAgree) {
    _listeners[_roomUuid]?.forEach((element) {
      element.onSeatInvitationAccepted(seatIndex, user, isAutoAgree);
    });
  }

  @override
  void onSeatInvitationCancelled(int seatIndex, String user, String operateBy) {
    _listeners[_roomUuid]?.forEach((element) {
      element.onSeatInvitationCancelled(seatIndex, user, operateBy);
    });
  }

  @override
  void onSeatInvitationReceived(int seatIndex, String user, String operateBy) {
    _listeners[_roomUuid]?.forEach((element) {
      element.onSeatInvitationReceived(seatIndex, user, operateBy);
    });
  }

  @override
  void onSeatInvitationRejected(int seatIndex, String user) {
    _listeners[_roomUuid]?.forEach((element) {
      element.onSeatInvitationRejected(seatIndex, user);
    });
  }

  @override
  void onSeatKicked(int seatIndex, String user, String operateBy) {
    _listeners[_roomUuid]?.forEach((element) {
      element.onSeatKicked(seatIndex, user, operateBy);
    });
  }

  @override
  void onSeatLeave(int seatIndex, String user) {
    _listeners[_roomUuid]?.forEach((element) {
      element.onSeatLeave(seatIndex, user);
    });
  }

  @override
  void onSeatListChanged(List<SeatItem?> seatItems) {
    _listeners[_roomUuid]?.forEach((element) {
      element.onSeatListChanged(seatItems);
    });
  }

  @override
  void onSeatManagerAdded(List<String?> managers) {
    _listeners[_roomUuid]?.forEach((element) {
      element.onSeatManagerAdded(managers);
    });
  }

  @override
  void onSeatManagerRemoved(List<String?> managers) {
    _listeners[_roomUuid]?.forEach((element) {
      element.onSeatManagerRemoved(managers);
    });
  }

  @override
  void onSeatRequestApproved(
      int seatIndex, String user, String operateBy, bool isAutoAgree) {
    _listeners[_roomUuid]?.forEach((element) {
      element.onSeatRequestApproved(seatIndex, user, operateBy, isAutoAgree);
    });
  }

  @override
  void onSeatRequestCancelled(int seatIndex, String user) {
    _listeners[_roomUuid]?.forEach((element) {
      element.onSeatRequestCancelled(seatIndex, user);
    });
  }

  @override
  void onSeatRequestRejected(int seatIndex, String user, String operateBy) {
    _listeners[_roomUuid]?.forEach((element) {
      element.onSeatRequestRejected(seatIndex, user, operateBy);
    });
  }

  @override
  void onSeatRequestSubmitted(int seatIndex, String user) {
    _listeners[_roomUuid]?.forEach((element) {
      element.onSeatRequestSubmitted(seatIndex, user);
    });
  }
}

class MethodChannelSeatController extends RoomSeatControllerPlatform {
  final _api = RoomSeatControllerApi();
  final _eventDispatcher = MethodChannelSeatEventSink();

  MethodChannelSeatController() {
    SeatEventSink.setup(_eventDispatcher);
  }

  @override
  Future<VoidResult> acceptSeatInvitation(String roomUuid) {
    return _api
        .acceptSeatInvitation(roomUuid)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> approveSeatRequest(String roomUuid, String user) {
    return _api
        .approveSeatRequest(roomUuid, user)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> cancelSeatInvitation(String roomUuid, String user) {
    return _api
        .cancelSeatInvitation(roomUuid, user)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> cancelSeatRequest(String roomUuid) {
    return _api
        .cancelSeatRequest(roomUuid)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> closeSeats(String roomUuid, List<int> seatIndices) {
    return _api
        .closeSeats(roomUuid, seatIndices)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<NEResult<NESeatInfo>> getSeatInfo(String roomUuid) {
    return _api.getSeatInfo(roomUuid).then((value) {
      List<NESeatItem>? list = [];
      value.seatInfo?.seatItems?.forEach((element) {
        list.add(NESeatItem(
            element?.index,
            element?.status,
            element?.user,
            element?.userName,
            element?.icon,
            element?.onSeatType,
            element?.updated));
      });
      List<String>? managers = [];
      value.seatInfo?.managers?.forEach((element) {
        managers.add(element ?? '');
      });
      NESeatInfo seatInfo = NESeatInfo(value.seatInfo?.creator, managers, list);
      return NEResult(code: value.code, msg: value.msg, data: seatInfo);
    });
  }

  @override
  Future<NEResult<List<NESeatInvitationItem>>> getSeatInvitationList(
      String roomUuid) {
    return _api.getSeatInvitationList(roomUuid).then((value) {
      List<NESeatInvitationItem>? list = [];
      value.seatInvitationList.forEach((element) {
        list.add(NESeatInvitationItem(element?.index, element?.user));
      });
      return NEResult(code: value.code, msg: value.msg, data: list);
    });
  }

  @override
  Future<NEResult<List<NESeatRequestItem>>> getSeatRequestList(
      String roomUuid) {
    return _api.getSeatRequestList(roomUuid).then((value) {
      List<NESeatRequestItem>? list = [];
      value.seatRequestList.forEach((element) {
        list.add(NESeatRequestItem(
            element?.index, element?.user, element?.userName, element?.icon));
      });
      return NEResult(code: value.code, msg: value.msg, data: list);
    });
  }

  @override
  Future<VoidResult> kickSeat(String roomUuid, String user) {
    return _api
        .kickSeat(roomUuid, user)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> leaveSeat(String roomUuid) {
    return _api
        .leaveSeat(roomUuid)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> openSeats(String roomUuid, List<int> seatIndices) {
    return _api
        .openSeats(roomUuid, seatIndices)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> rejectSeatInvitation(String roomUuid) {
    return _api
        .rejectSeatInvitation(roomUuid)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> rejectSeatRequest(String roomUuid, String user) {
    return _api
        .rejectSeatRequest(roomUuid, user)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> sendSeatInvitation(
      String roomUuid, int seatIndex, String user) {
    return _api
        .sendSeatInvitation(roomUuid, seatIndex, user)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> submitSeatRequest(
      String roomUuid, int seatIndex, bool exclusive) {
    return _api
        .submitSeatRequest(roomUuid, seatIndex, exclusive)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  void registerEventListener(String roomUuid, SeatEventSink listener) {
    _eventDispatcher.registerEventListener(roomUuid, listener);
  }

  @override
  void unregisterEventListener(String roomUuid, SeatEventSink listener) {
    _eventDispatcher.unregisterEventListener(roomUuid, listener);
  }
}
