// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import 'package:netease_common/netease_common.dart';
import 'package:netease_roomkit_interface/src/pigeon.dart';
import 'package:netease_roomkit_interface/src/platform_interface/room_waiting_room_controller_platform.dart';

import '../types/waiting_room.dart';

class MethodChannelWaitingRoomController
    extends RoomWaitingRoomControllerPlatform implements WaitingRoomEventSink {
  final _listeners = <String, Set<WaitingRoomEventSink>>{};
  final _api = WaitingRoomControllerApi();

  MethodChannelWaitingRoomController() {
    WaitingRoomEventSink.setup(this);
  }

  @override
  Future<VoidResult> admitMember(String roomUuid, String userUuid) {
    return _api
        .admitMember(roomUuid, userUuid)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> changeMemberName(
      String roomUuid, String userUuid, String name) {
    return _api
        .changeMemberName(roomUuid, userUuid, name)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> disableWaitingRoomOnEntry(String roomUuid, bool admitAll) {
    return _api
        .disableWaitingRoomOnEntry(roomUuid, admitAll)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> enableWaitingRoomOnEntry(String roomUuid) {
    return _api
        .enableWaitingRoomOnEntry(roomUuid)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> expelMember(
      String roomUuid, String userUuid, bool disallowRejoin) {
    return _api
        .expelMember(roomUuid, userUuid, disallowRejoin)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<NEResult<List<NEWaitingRoomMember>>> getMemberList(
      String roomUuid, int joinTime, int size, bool asc) {
    return _api.getMemberList(roomUuid, joinTime, size, asc).then(
          (value) => NEResult(
            code: value.code,
            msg: value.msg,
            data: value.members
                ?.whereType<WaitingRoomMember>()
                .map((e) => NEWaitingRoomMember(
                      uuid: e.uuid,
                      name: e.name,
                      avatar: e.avatar,
                      joinTime: e.joinTime,
                      status: e.status,
                    ))
                .toList(),
          ),
        );
  }

  @override
  Future<VoidResult> putInWaitingRoom(String roomUuid, String userUuid) {
    return _api
        .putInWaitingRoom(roomUuid, userUuid)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  void registerEventListener(String roomUuid, WaitingRoomEventSink listener) {
    _listeners
        .putIfAbsent(roomUuid, () => <WaitingRoomEventSink>{})
        .add(listener);
  }

  @override
  void unregisterEventListener(String roomUuid, WaitingRoomEventSink listener) {
    _listeners[roomUuid]?.remove(listener);
  }

  @override
  void onMemberAdmitted(String roomUuid, String member) {
    _listeners[roomUuid]?.forEach((element) {
      element.onMemberAdmitted(roomUuid, member);
    });
  }

  @override
  void onMemberJoin(String roomUuid, WaitingRoomMember member, int reason) {
    _listeners[roomUuid]?.forEach((element) {
      element.onMemberJoin(roomUuid, member, reason);
    });
  }

  @override
  void onMemberLeave(String roomUuid, String member, int reason) {
    _listeners[roomUuid]?.forEach((element) {
      element.onMemberLeave(roomUuid, member, reason);
    });
  }

  @override
  void onMemberNameChanged(
      String roomUuid, String member, String name, RoomMember? operateBy) {
    _listeners[roomUuid]?.forEach((element) {
      element.onMemberNameChanged(roomUuid, member, name, operateBy);
    });
  }

  @override
  void onMyWaitingRoomStatusChanged(String roomUuid, int status, int reason) {
    _listeners[roomUuid]?.forEach((element) {
      element.onMyWaitingRoomStatusChanged(roomUuid, status, reason);
    });
  }

  @override
  void onWaitingRoomInfoUpdated(String roomUuid, WaitingRoomInfo info) {
    _listeners[roomUuid]?.forEach((element) {
      element.onWaitingRoomInfoUpdated(roomUuid, info);
    });
  }
}
