// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import 'dart:convert';

import 'package:json_annotation/json_annotation.dart';

part 'room_service_models.g.dart';

///
/// 创建房间参数
///
@JsonSerializable(
  createToJson: false,
)
class NECreateRoomParams {
  final String roomUuid;

  final String roomName;

  final String? password;

  final int templateId;

  @JsonKey(
    fromJson: _seatInitParamsFromJson,
  )
  final NESeatInitParams? seatInitParams;

  @JsonKey(
    fromJson: _mapCast,
  )
  final Map<String, String>? initialRoomProperties;

  NECreateRoomParams({
    required this.roomUuid,
    required this.roomName,
    required this.templateId,
    this.password,
    this.seatInitParams,
    this.initialRoomProperties,
  });

  factory NECreateRoomParams.fromJson(Map map) {
    return _$NECreateRoomParamsFromJson(map.cast());
  }

  @override
  String toString() {
    return 'NECreateRoomParams{roomUuid: $roomUuid, roomName: $roomName, password: $password, templateId: $templateId, seatInitParams: $seatInitParams, initialRoomProperties: $initialRoomProperties}';
  }
}

/// 麦位配置信息，用于创建房间时进行麦位初始化。
/// @property seatCount 麦位数量
/// @property seatRequestApprovalMode 麦位申请审批模式, 参考[NESeatRequestApprovalMode]
/// @property seatInvitationConfirmMode 麦位邀请确认模式, 参考[NESeatInvitationConfirmMode]
@JsonSerializable(
  createToJson: false,
)
class NESeatInitParams {
  final int seatCount;
  final int seatRequestApprovalMode;
  final int seatInvitationConfirmMode;

  NESeatInitParams(this.seatCount, this.seatRequestApprovalMode,
      this.seatInvitationConfirmMode);

  factory NESeatInitParams.fromJson(Map json) {
    return _$NESeatInitParamsFromJson(json.cast());
  }

  @override
  String toString() {
    return 'NESeatInitParams{seatCount: $seatCount, seatRequestApprovalMode: $seatRequestApprovalMode, seatInvitationConfirmMode: $seatInvitationConfirmMode}';
  }
}

NESeatInitParams? _seatInitParamsFromJson(Map? json) {
  return json != null ? NESeatInitParams.fromJson(json) : null;
}

@JsonSerializable(
  createToJson: false,
)
class NECreateRoomOptions {
  final bool enableRecord;
  final bool enableLive;
  final bool enableRtc;
  final bool enableChatroom;
  final bool enableWhiteboard;
  final bool enableSip;
  final bool enableWaitingRoom;

  NECreateRoomOptions({
    this.enableRecord = false,
    this.enableLive = false,
    this.enableRtc = true,
    this.enableChatroom = false,
    this.enableWhiteboard = false,
    this.enableSip = false,
    this.enableWaitingRoom = false,
  });

  factory NECreateRoomOptions.fromJson(Map map) {
    return _$NECreateRoomOptionsFromJson(map.cast());
  }

  @override
  String toString() {
    return 'NECreateRoomOptions{enableRecord: $enableRecord, enableLive: $enableLive, enableRtc: $enableRtc, enableChatroom: $enableChatroom, enableWhiteboard: $enableWhiteboard, enableSip: $enableSip}';
  }
}

///
/// 跨应用加房间所需的鉴权信息，包含房间所属的AppKey以及身份验证信息。
///
class NEInjectedAuthorization {
  final String appKey;
  final String user;
  final String token;

  NEInjectedAuthorization({
    required this.appKey,
    required this.user,
    required this.token,
  }) : assert(appKey.isNotEmpty && user.isNotEmpty && token.isNotEmpty);

  factory NEInjectedAuthorization.fromJson(Map map) {
    return NEInjectedAuthorization(
      appKey: map['appKey'] as String,
      user: map['user'] as String,
      token: map['token'] as String,
    );
  }
}

///
/// 加入房间参数
///
@JsonSerializable(
  createToJson: false,
)
class NEJoinRoomParams {
  ///
  /// 房间id
  ///
  final String roomUuid;

  ///
  /// 用户名
  ///
  final String userName;

  ///
  /// 用户头像 URL 地址
  ///
  final String? avatar;

  ///
  /// 角色
  ///
  final String role;

  ///
  /// 密码
  ///
  final String? password;

  ///
  /// 开启/关闭代理模式。需要配合 [injectedAuthorization] 使用，否则无效。
  /// 在代理模式下，代理人以自己的身份加入房间，
  /// 但在房间内发起的会控操作会以 [injectedAuthorization] 指定的被代理人的身份执行。
  ///
  final bool agentMode;

  ///
  /// 房间所需的鉴权信息
  ///
  final NEInjectedAuthorization? injectedAuthorization;

  ///
  /// 初始化属性
  ///
  @JsonKey(
    fromJson: _mapCast,
  )
  final Map<String, String>? initialMyProperties;

  NEJoinRoomParams({
    required this.roomUuid,
    required this.userName,
    this.avatar,
    required this.role,
    this.password,
    this.agentMode = false,
    this.injectedAuthorization,
    this.initialMyProperties,
  }) : assert(!agentMode || injectedAuthorization != null);

  factory NEJoinRoomParams.fromJson(Map map) {
    return _$NEJoinRoomParamsFromJson(map.cast());
  }

  @override
  String toString() {
    return 'NEJoinRoomParams{roomUuid: $roomUuid, userName: $userName, role: $role, avatar: $avatar, password: $password, initialProperties: $initialMyProperties}';
  }
}

///
/// 加入房间选项
///
@JsonSerializable(
  createToJson: false,
)
class NEJoinRoomOptions {
  ///
  /// 配置在加入Rtc频道成功后是否打开本地音频设备，默认不打开。
  /// 该选项若配置为打开，则SDK会提前初始化音频设备，但不会对外发送本地音频流。
  /// 如果需要发送音频流，还需要调用 [unmuteMyAudio] 方法。
  ///
  final bool enableMyAudioDeviceOnJoinRtc;

  /// 是否自动订阅
  final bool autoSubscribeAudio;

  NEJoinRoomOptions({
    this.enableMyAudioDeviceOnJoinRtc = false,
    this.autoSubscribeAudio = true,
  });

  factory NEJoinRoomOptions.fromJson(Map map) {
    return _$NEJoinRoomOptionsFromJson(map.cast());
  }
}

///
/// 预览房间参数
///
class NEPreviewRoomParams {}

///
/// 预览房间选项
///
class NEPreviewRoomOptions {}

///
/// 房间角色
///
class NERoomRole {
  final String name;
  final int limit;
  final bool hide;
  final NERoleParams? params;

  NERoomRole({
    required this.name,
    required this.limit,
    required this.hide,
    required this.params,
  });

  Map toJson() => {
        'name': name,
        'limit': limit,
        'hide': hide,
        if (params != null) 'params': params!.toJson(),
      };

  @override
  String toString() {
    return jsonEncode(toJson());
  }
}

///
/// 角色音视频参数
///
class NERoleParams {
  final NERoleVideoParams? video;

  final NERoleAudioParams? audio;

  final NERoleScreenParams? screen;

  NERoleParams({
    required this.video,
    required this.audio,
    required this.screen,
  });

  Map toJson() => {
        if (video != null) 'video': video!.toJson(),
        if (audio != null) 'audio': audio!.toJson(),
        if (screen != null) 'screen': screen!.toJson(),
      };
}

///
/// 角色所对应的视频参数
/// [width] 视频宽
/// [height] 视频高
/// [fps] 帧率
///
class NERoleVideoParams {
  final int width;

  final int height;

  final int fps;

  NERoleVideoParams({
    required this.width,
    required this.height,
    required this.fps,
  });

  Map toJson() => {
        'width': width,
        'height': height,
        'fps': fps,
      };
}

///
/// 角色所对应的屏幕共享参数
///
class NERoleScreenParams {
  final String videoProfile;
  final int fps;

  NERoleScreenParams(this.videoProfile, this.fps);

  Map toJson() => {
        'videoProfile': videoProfile,
        'fps': fps,
      };
}

///
/// 角色所对应的音频参数
///
class NERoleAudioParams {
  final String profile;
  final String scenario;

  NERoleAudioParams(this.profile, this.scenario);

  Map toJson() => {
        'profile': profile,
        'scenario': scenario,
      };
}

Map<String, String>? _mapCast(Map? map) {
  return map?.cast();
}
