// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import 'package:json_annotation/json_annotation.dart';
part 'rtc_models.g.dart';

///
/// 房间音频dump文件类型
///
enum NEAudioDumpType {
  kPCM,
  kAll,
  kWAV,
}

///
/// 本地音频输出设备
///
enum NEAudioOutputDevice {
  ///
  /// 扬声器
  ///
  kSpeakerPhone,

  ///
  /// 有线耳机
  ///
  kWiredHeadset,

  ///
  /// 听筒
  ///
  kEarpiece,

  ///
  /// 蓝牙耳机
  ///
  kBluetoothHeadset,
}

extension NEAudioOutputDeviceTypeExtension on NEAudioOutputDevice {
  static NEAudioOutputDevice fromType(int type) {
    return NEAudioOutputDevice.values.firstWhere(
      (element) => element.index == type,
      orElse: () => NEAudioOutputDevice.kEarpiece,
    );
  }
}

@JsonSerializable()
class NECreateAudioMixingOption {
  String path;
  int loopCount;
  bool sendEnabled;
  bool playbackEnabled;
  int sendVolume;
  int playbackVolume;
  int startTimeStamp;
  NERoomRtcAudioStreamType sendWithAudioType;

  NECreateAudioMixingOption({
    required this.path,
    this.loopCount = 0,
    this.sendEnabled = true,
    this.playbackEnabled = true,
    this.sendVolume = 100,
    this.playbackVolume = 100,
    this.startTimeStamp = 0,
    this.sendWithAudioType = NERoomRtcAudioStreamType.kNERtcAudioStreamTypeMain,
  });

  factory NECreateAudioMixingOption.fromJson(Map<String, dynamic> json) {
    return _$NECreateAudioMixingOptionFromJson(json);
  }

  Map toJson() => _$NECreateAudioMixingOptionToJson(this);
}

@JsonSerializable()
class NECreateAudioEffectOption {
  String path;
  int loopCount;
  bool sendEnabled;
  bool playbackEnabled;
  int sendVolume;
  int playbackVolume;
  int startTimeStamp;
  int progressInterval;
  NERoomRtcAudioStreamType sendWithAudioType;

  NECreateAudioEffectOption({
    required this.path,
    this.loopCount = 0,
    this.sendEnabled = true,
    this.playbackEnabled = true,
    this.sendVolume = 100,
    this.playbackVolume = 100,
    this.startTimeStamp = 0,
    this.progressInterval = 0,
    this.sendWithAudioType = NERoomRtcAudioStreamType.kNERtcAudioStreamTypeMain,
  });
  factory NECreateAudioEffectOption.fromJson(Map<String, dynamic> json) {
    return _$NECreateAudioEffectOptionFromJson(json);
  }

  Map toJson() => _$NECreateAudioEffectOptionToJson(this);
}

enum NERoomBeautyEffectType {
  /// -1
  kUnknownType,
  kWhiteTeeth,
  kLightEye,
  kWhiten,
  kSmooth,
  kSmallNose,
  kEyeDis,
  kEyeAngle,
  kMouth,
  kBigEye,
  kSmallFace,
  kJaw,
  kThinFace,
  kFaceRuddy,
  kLongNose,
  kPhiltrum,
  kMouthAngle,
  kRoundEye,
  kEyeCorner,
  kVFace,
  kUnderJaw,
  kNarrowFace,
  kCheekBone,
  kFaceSharpen,
}

enum NEEncryptionMode {
  GMCryptoSM4ECB,
}

///
/// 订阅视频流类型
///
enum NEVideoStreamType {
  ///
  /// 大流
  ///
  kHigh,

  ///
  /// 小流
  ///
  kLow
}

/// 音频流类型
enum NERoomRtcAudioStreamType {
  /// 主流
  kNERtcAudioStreamTypeMain,

  /// 辅流
  kNERtcAudioStreamTypeSub
}

extension NERoomBeautyEffectTypeExtension on NERoomBeautyEffectType {
  static int fromType(NERoomBeautyEffectType beautyType) =>
      (NERoomBeautyEffectType.values
          .firstWhere(
            (element) => element == beautyType,
            orElse: () => NERoomBeautyEffectType.kUnknownType,
          )
          .index);
}

class NERoomVirtualBackgroundType {
  static const kBackgroundColor = 1;
  static const kBackgroundImg = 2;
  static const kBackgroundBlur = 3;

  static const kBlurDegreeLow = 1;
  static const kBlurDegreeMedium = 2;
  static const kBlurDegreeHigh = 3;
}

@JsonSerializable()
class NERoomRtcStats {
  // 发送字节数（Byte），累计值。
  int txBytes;

  /// 接收字节数（Byte），累计值。
  int rxBytes;

  /// 当前 App 的 CPU 使用率 (%)。Android 8.0及后续版本无法获取
  int cpuAppUsage;

  /// 当前系统的 CPU 使用率 (%)。Android 8.0及后续版本无法获取。
  int cpuTotalUsage;

  /// 当前 App 的内存占比 (%) , 占最大可用内存，由于计算方式的原因可能会超过100。
  int memoryAppUsageRatio;

  /// 当前系统的内存占比 (%)。
  int memoryTotalUsageRatio;

  /// 当前 App 的内存大小 (KB)。
  int memoryAppUsageInKBytes;

  /// 自加入房间的通话时长 ， 退出后再加入重新计时 ( 单位：S)。
  int totalDuration;

  ///  自加入房间后累计的发送的音频字节数（Byte）。
  int txAudioBytes;

  /// 自加入房间后累计的发送的视频字节数（Byte）。
  int txVideoBytes;

  /// 自加入房间后累计的接收的音频字节数（Byte）。
  int rxAudioBytes;

  /// 自加入房间后累计的接收的视频字节数（Byte）。
  int rxVideoBytes;

  /// 音频接收码率（kbps）。
  int rxAudioKBitRate;

  /// 视频接收码率（kbps）。
  int rxVideoKBitRate;

  /// 音频发送码率（kbps）。
  int txAudioKBitRate;

  /// 视频发送码率（kbps）。
  int txVideoKBitRate;

  /// 上行平均往返时延(ms)。
  int upRtt;

  /// 下行平均往返时延(ms)。
  int downRtt;

  /// 本地上行音频丢包率(%)。
  int txAudioPacketLossRate;

  /// 本地上行视频实际丢包率(%)。
  int txVideoPacketLossRate;

  /// 本地上行音频丢包数。
  int txAudioPacketLossSum;

  ///  本地上行视频丢包数。
  int txVideoPacketLossSum;

  /// 本地上行音频抖动 (ms)。
  int txAudioJitter;

  /// 本地上行视频抖动 (ms)。
  int txVideoJitter;

  /// 本地下行音频丢包率(%)。
  int rxAudioPacketLossRate;

  /// 本地下行视频丢包率(%)。
  int rxVideoPacketLossRate;

  /// 本地下行音频丢包数。
  int rxAudioPacketLossSum;

  /// 本地下行视频丢包数。
  int rxVideoPacketLossSum;

  /// 本地下行音频抖动 (ms)。
  int rxAudioJitter;

  /// 本地下行视频抖动 (ms)。
  int rxVideoJitter;

  NERoomRtcStats(
      {this.txBytes = 0,
      this.rxBytes = 0,
      this.cpuAppUsage = 0,
      this.cpuTotalUsage = 0,
      this.memoryAppUsageRatio = 0,
      this.memoryTotalUsageRatio = 0,
      this.memoryAppUsageInKBytes = 0,
      this.totalDuration = 0,
      this.txAudioBytes = 0,
      this.txVideoBytes = 0,
      this.rxAudioBytes = 0,
      this.rxVideoBytes = 0,
      this.rxAudioKBitRate = 0,
      this.rxVideoKBitRate = 0,
      this.txAudioKBitRate = 0,
      this.txVideoKBitRate = 0,
      this.upRtt = 0,
      this.downRtt = 0,
      this.txAudioPacketLossRate = 0,
      this.txVideoPacketLossRate = 0,
      this.txAudioPacketLossSum = 0,
      this.txVideoPacketLossSum = 0,
      this.txAudioJitter = 0,
      this.txVideoJitter = 0,
      this.rxAudioPacketLossRate = 0,
      this.rxVideoPacketLossRate = 0,
      this.rxAudioPacketLossSum = 0,
      this.rxVideoPacketLossSum = 0,
      this.rxAudioJitter = 0,
      this.rxVideoJitter = 0});

  factory NERoomRtcStats.fromJson(Map<String, dynamic> json) {
    return _$NERoomRtcStatsFromJson(json);
  }

  Map toJson() => _$NERoomRtcStatsToJson(this);
}

enum NERoomRtcNetworkStatusType {
  kStatusUnknown,
  kStatusExcellent,
  kStatusGood,
  kStatusPoor,
  kStatusBad,
  kStatusVeryBad,
  kStatusDown
}

extension NERoomRtcNetworkStatusTypeExtension on NERoomRtcNetworkStatusType {
  static NERoomRtcNetworkStatusType getTypeValue(int index) {
    switch (index) {
      case 1:
        return NERoomRtcNetworkStatusType.kStatusExcellent;
      case 2:
        return NERoomRtcNetworkStatusType.kStatusGood;
      case 3:
        return NERoomRtcNetworkStatusType.kStatusPoor;
      case 4:
        return NERoomRtcNetworkStatusType.kStatusBad;
      case 5:
        return NERoomRtcNetworkStatusType.kStatusVeryBad;
      case 6:
        return NERoomRtcNetworkStatusType.kStatusDown;
      default:
        return NERoomRtcNetworkStatusType.kStatusUnknown;
    }
  }
}

@JsonSerializable()
class NERoomRtcNetworkQualityInfo {
  String? userId;
  NERoomRtcNetworkStatusType upStatus;
  NERoomRtcNetworkStatusType downStatus;

  NERoomRtcNetworkQualityInfo(this.userId,
      {this.upStatus = NERoomRtcNetworkStatusType.kStatusUnknown,
      this.downStatus = NERoomRtcNetworkStatusType.kStatusUnknown});

  factory NERoomRtcNetworkQualityInfo.fromJson(Map<String, dynamic> json) {
    return _$NERoomRtcNetworkQualityInfoFromJson(json);
  }

  Map toJson() => _$NERoomRtcNetworkQualityInfoToJson(this);
}
