// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

///
/// 等候室成员对象
///
class NEWaitingRoomMember {
  ///
  /// 用户id
  ///
  final String uuid;

  ///
  /// 用户名
  ///
  String name;

  ///
  /// 用户头像
  ///
  String? avatar;

  ///
  /// 成员加入等候室的服务端时间戳，可用于计算等待时间或排序
  ///
  int joinTime;

  ///
  /// 成员当前状态，参考[NEWaitingRoomConstants]
  ///
  int status;

  NEWaitingRoomMember({
    required this.uuid,
    required this.name,
    this.avatar,
    this.joinTime = 0,
    required this.status,
  });
}

///
/// 等候室房间信息
///
class NEWaitingRoomInfo {
  ///
  /// 当前等候室成员数量
  ///
  final int memberCount;

  ///
  /// 当前房间是否开启了等候室
  ///
  final bool isEnabledOnEntry;

  ///
  /// 等候室背景图
  ///
  final String? backgroundImageUrl;

  const NEWaitingRoomInfo({
    required this.memberCount,
    required this.isEnabledOnEntry,
    this.backgroundImageUrl,
  });

  NEWaitingRoomInfo copyWith({
    int? memberCount,
    bool? isEnabledOnEntry,
    String? backgroundImageUrl,
  }) {
    return NEWaitingRoomInfo(
      memberCount: memberCount ?? this.memberCount,
      isEnabledOnEntry: isEnabledOnEntry ?? this.isEnabledOnEntry,
      backgroundImageUrl: backgroundImageUrl ?? this.backgroundImageUrl,
    );
  }

  @override
  bool operator ==(Object other) =>
      identical(this, other) ||
      other is NEWaitingRoomInfo &&
          runtimeType == other.runtimeType &&
          memberCount == other.memberCount &&
          isEnabledOnEntry == other.isEnabledOnEntry &&
          backgroundImageUrl == other.backgroundImageUrl;

  @override
  int get hashCode =>
      Object.hash(memberCount, isEnabledOnEntry, backgroundImageUrl);
}

class NEWaitingRoomConstants {
  ///
  /// 等候中
  ///
  static const STATUS_WAITING = 1;

  ///
  /// 被管理员准入（正在加入）
  ///
  static const STATUS_ADMITTED = 2;

  ///
  /// 已离开
  ///
  static const STATUS_LEFT = 3;

  ///
  /// 无效
  ///
  static const REASON_INVALID = -1;

  ///
  /// 自动加入/自动离开等候室
  ///
  static const REASON_BY_SELF = 0;

  ///
  /// 被管理员移至等候室
  ///
  static const REASON_PUTTED_IN = 1;

  ///
  /// 同账号互踢
  ///
  static const REASON_SELF_KICKED = 2;

  ///
  /// 被管理员移除等候室
  ///
  static const REASON_EXPELLED = 3;

  ///
  /// 超时
  ///
  static const REASON_TIMEOUT = 4;

  ///
  /// 已经加入房间
  ///
  static const REASON_HAS_JOINED = 5;
}

extension NEWaitingRoomStatusStringify on int {
  String toWaitingRoomStatusString() {
    switch (this) {
      case NEWaitingRoomConstants.STATUS_WAITING:
        return 'waiting';
      case NEWaitingRoomConstants.STATUS_ADMITTED:
        return 'admitted';
      case NEWaitingRoomConstants.STATUS_LEFT:
        return 'left';
      default:
        return 'unknown';
    }
  }
}

extension NEWaitingRoomReasonStringify on int {
  String toWaitingRoomReasonString() {
    switch (this) {
      case NEWaitingRoomConstants.REASON_INVALID:
        return 'invalid';
      case NEWaitingRoomConstants.REASON_BY_SELF:
        return 'bySelf';
      case NEWaitingRoomConstants.REASON_PUTTED_IN:
        return 'puttedIn';
      case NEWaitingRoomConstants.REASON_SELF_KICKED:
        return 'selfKicked';
      case NEWaitingRoomConstants.REASON_EXPELLED:
        return 'expelled';
      case NEWaitingRoomConstants.REASON_TIMEOUT:
        return 'timeout';
      case NEWaitingRoomConstants.REASON_HAS_JOINED:
        return 'hasJoined';
      default:
        return 'unknown';
    }
  }
}
