// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import 'package:pigeon/pigeon.dart';

class SimpleResponse {
  final int code;

  final String? msg;

  final int? data1;

  final int? data2;

  final String? data3;

  SimpleResponse(this.code, this.msg, this.data1, this.data2, this.data3);
}

// **************************************************************************

class InitializeRequest {
  final String appKey;

  final String? serverUrl;

  final bool useAssetServerConfig;

  final String? serverConfigJson;

  final Map<String?, String?>? extras;

  InitializeRequest({
    required this.appKey,
    this.serverUrl,
    this.useAssetServerConfig = false,
    this.serverConfigJson,
    this.extras,
  });
}

class SDKVersions {
  String? imVersion;
  String? rtcVersion;
  String? whiteboardVersion;
  String? roomKitVersion;
}

@HostApi(dartHostTestHandler: "RoomKitApi")
abstract class RoomKitApi {
  @async
  SimpleResponse initialize(InitializeRequest options);

  SDKVersions sdkVersions();

  int switchLanguage(String languageCode);

  String getDeviceId();

  @async
  String? uploadLog();

  @async
  Object? invokeMethod(String method, Map<String, Object>? arguments);
}

// **************************************************************************

@HostApi(dartHostTestHandler: "AuthServiceApi")
abstract class AuthServiceApi {
  @async
  SimpleResponse login(String account, String token);

  @async
  SimpleResponse logout();

  bool isLoggedIn();
}

class AuthStatusEvent {
  final String status;
  final bool isLoggedIn;

  AuthStatusEvent(this.status, this.isLoggedIn);
}

@FlutterApi()
abstract class AuthStatusEventSink {
  void onAuthStatusChanged(AuthStatusEvent event);
}

// **************************************************************************
class CreateRoomParams {
  final String roomUuid;
  final String roomName;
  final int templateId;
  final String? password;
  final SeatInitParams? seatInitParams;
  final Map<String?, String?>? initialProperties;

  CreateRoomParams({
    required this.roomUuid,
    required this.roomName,
    required this.templateId,
    this.password,
    this.seatInitParams,
    this.initialProperties,
  });
}

class SeatInitParams {
  final int seatCount;
  final int seatRequestApprovalMode;
  final int seatInvitationConfirmMode;

  SeatInitParams(this.seatCount, this.seatRequestApprovalMode,
      this.seatInvitationConfirmMode);
}

class CreateRoomOptions {
  final bool enableRecord;
  final bool enableLive;
  final bool enableRtc;
  final bool enableChatroom;
  final bool enableWhiteboard;
  final bool enableWaitingRoom;
  // final bool enableSip;

  CreateRoomOptions({
    this.enableRecord = false,
    this.enableLive = false,
    this.enableRtc = true,
    this.enableChatroom = true,
    this.enableWhiteboard = false,
    this.enableWaitingRoom = false,
    // this.enableSip = false,
  });
}

class InjectedAuthorization {
  final String appKey;
  final String user;
  final String token;

  InjectedAuthorization(
    this.appKey,
    this.user,
    this.token,
  );
}

class JoinRoomParams {
  final String roomUuid;
  final String userName;
  final String? avatar;
  final String role;
  final String? password;
  final bool agentMode;
  final InjectedAuthorization? injectedAuthorization;
  final Map<String?, String?>? initialProperties;

  JoinRoomParams({
    required this.roomUuid,
    required this.userName,
    required this.avatar,
    required this.role,
    this.password,
    required this.agentMode,
    this.injectedAuthorization,
    this.initialProperties,
  });
}

class JoinRoomOptions {
  final bool enableMyAudioDeviceOnJoinRtc;
  final bool autoSubscribeAudio;

  JoinRoomOptions(this.enableMyAudioDeviceOnJoinRtc, this.autoSubscribeAudio);
}

class PreviewRoomParams {
  // 占位，解决编译问题
  int? value;

  PreviewRoomParams();
}

class PreviewRoomOptions {
  // 占位，解决编译问题
  int? value;

  PreviewRoomOptions();
}

class RoomRole {
  final String name;
  final int limit;
  final bool hide;
  final RoleParams? params;

  RoomRole({
    required this.name,
    required this.limit,
    this.hide = false,
    required this.params,
  });
}

class RoleParams {
  final RoleVideoParams? video;
  final RoleAudioParams? audio;
  final RoleScreenParams? screen;

  RoleParams({
    required this.video,
    required this.audio,
    required this.screen,
  });
}

class RoleVideoParams {
  final int width;
  final int height;
  final int fps;

  RoleVideoParams({
    required this.width,
    required this.height,
    required this.fps,
  });
}

class RoleScreenParams {
  final String videoProfile;
  final int fps;

  RoleScreenParams(this.videoProfile, this.fps);
}

class RoleAudioParams {
  final String profile;
  final String scenario;

  RoleAudioParams(this.profile, this.scenario);
}

class RoomMember {
  final String uuid;
  final String name;
  final String? avatar;
  final RoomRole? role;
  final bool isAudioOn;
  final bool isVideoOn;
  final bool isInRtcChannel;
  final bool isInChatroom;
  final bool isSharingScreen;
  final bool isSharingWhiteboard;
  final bool isAudioConnected;
  final Map<String?, String?>? properties;
  final String clientType;

  RoomMember({
    required this.uuid,
    required this.name,
    required this.avatar,
    required this.role,
    required this.isAudioOn,
    required this.isVideoOn,
    required this.isInRtcChannel,
    required this.isInChatroom,
    required this.isSharingScreen,
    required this.isSharingWhiteboard,
    required this.isAudioConnected,
    required this.properties,
    required this.clientType,
  });
}

class RoomSnapshot {
  String roomUuid;
  String roomName;
  String? password;
  bool isRoomLocked;
  int rtcStartTime;
  RoomMember? localMember;
  List<RoomMember?> remoteMembers;
  Map<String?, String?>? properties;
  bool isRtcSupported;
  bool isChatroomSupported;
  bool isWhiteboardSupported;
  bool isCloudRecordSupported;
  bool isLiveSupported;
  bool isSeatSupported;
  String? sipCid;
  int remainingSeconds;
  bool isCloudRecording;
  bool? isWaitingRoomSupported;
  bool? isInWaitingRoom;
  WaitingRoomInfo? waitingRoomInfo;

  RoomSnapshot({
    required this.roomUuid,
    required this.roomName,
    required this.password,
    required this.rtcStartTime,
    required this.localMember,
    required this.remoteMembers,
    required this.properties,
    required this.isRtcSupported,
    required this.isChatroomSupported,
    required this.isWhiteboardSupported,
    required this.isCloudRecordSupported,
    required this.isLiveSupported,
    required this.isSeatSupported,
    required this.isRoomLocked,
    required this.sipCid,
    required this.remainingSeconds,
    required this.isCloudRecording,
    this.isWaitingRoomSupported = false,
    this.isInWaitingRoom = false,
    this.waitingRoomInfo,
  });
}

class JoinRoomResponse {
  final int code;

  final String? msg;

  final RoomSnapshot? snapshot;

  JoinRoomResponse(this.code, this.msg, this.snapshot);
}

class RecordListResponse {
  final int code;

  final String? msg;

  final List<RoomRecord?>? recordList;

  RecordListResponse(this.code, this.msg, this.recordList);
}

class RecordFileInfo {
  final String type;
  final bool mix;
  final String filename;
  final String md5;
  final int size;
  final String url;
  final int vid;
  final int pieceIndex;
  final String? userUuid;
  final String? nickname;

  RecordFileInfo({
    required this.type,
    required this.mix,
    required this.filename,
    required this.md5,
    required this.size,
    required this.url,
    required this.vid,
    required this.pieceIndex,
    this.userUuid,
    this.nickname,
  });
}

class RoomRecord {
  final String recordId;
  final int recordStartTime;
  final int recordEndTime;
  final List<RecordFileInfo?> infoList;

  RoomRecord({
    required this.recordId,
    required this.recordStartTime,
    required this.recordEndTime,
    required this.infoList,
  });
}

@HostApi(dartHostTestHandler: "RoomServiceApiOfDart")
abstract class RoomServiceApi {
  @async
  SimpleResponse createRoom(CreateRoomParams params, CreateRoomOptions options);

  @async
  JoinRoomResponse joinRoom(JoinRoomParams params, JoinRoomOptions options);

  @async
  SimpleResponse previewRoom(
      PreviewRoomParams params, PreviewRoomOptions options);

  @async
  RecordListResponse getRoomCloudRecordList(String roomArchiveId);

  @async
  ChatRoomMessagesResponse fetchChatroomHistoryMessages(
      String roomArchiveId, ChatroomHistoryMessageSearchOption option);

  @async
  SimpleResponse downloadAttachment(String messageUuid);
}

@HostApi(dartHostTestHandler: "RoomContextApiOfDart")
abstract class RoomContextApi {
  @async
  SimpleResponse changeMyName(String roomUuid, String name);

  @async
  SimpleResponse changeMemberName(
      String roomUuid, String userUuid, String name);

  @async
  SimpleResponse changeMemberRole(
      String roomUuid, String userUuid, String role);

  @async
  SimpleResponse handOverMyRole(String roomUuid, String userUuid);

  @async
  SimpleResponse kickMemberOut(String roomUuid, String userUuid);

  @async
  SimpleResponse leaveRoom(String roomUuid);

  @async
  SimpleResponse endRoom(String roomUuid, bool isForce);

  @async
  SimpleResponse updateRoomProperty(
      String roomUuid, String key, String value, String? associatedUuid);

  @async
  SimpleResponse deleteRoomProperty(String roomUuid, String key);

  @async
  SimpleResponse updateMemberProperty(
      String roomUuid, String userUuid, String key, String value);

  @async
  SimpleResponse deleteMemberProperty(
      String roomUuid, String userUuid, String key);

  @async
  SimpleResponse lockRoom(String roomUuid);

  @async
  SimpleResponse unlockRoom(String roomUuid);

  @async
  SimpleResponse startCloudRecord(String roomUuid);

  @async
  SimpleResponse stopCloudRecord(String roomUuid);

  @async
  JoinRoomResponse rejoinAfterAdmittedToRoom(String roomUuid);
}

class RoomMessageAttachment {
  final String? displayName;

  final String? extension;

  final String? md5;

  final String url;

  final int size;

  final String? thumbPath;

  final String? path;

  final int? width;

  final int? height;

  RoomMessageAttachment(
    this.displayName,
    this.extension,
    this.md5,
    this.url,
    this.size,
    this.thumbPath,
    this.path,
    this.width,
    this.height,
  );
}

enum RoomEventType { recall, undefined }

enum ChatroomType { common, waitingRoom }

class RoomMessage {
  final String uuid;

  final String type;

  final String? fromAccount;

  final String fromNick;

  final String? fromAvatar;

  final String? content;

  final List<String?>? toAccounts;

  final int time;

  final RoomMessageAttachment? attachment;

  final int? eventType;
  final String? operatorAccount;
  final String? recallMessageId;
  final ChatroomType chatroomType;
  RoomMessage(
    this.uuid,
    this.type,
    this.fromAccount,
    this.fromNick,
    this.fromAvatar,
    this.content,
    this.toAccounts,
    this.time,
    this.attachment,
    this.eventType,
    this.operatorAccount,
    this.recallMessageId,
    this.chatroomType,
  );
}

class RoomMessageEvent {
  final String roomUuid;
  final List<RoomMessage?> messages;

  RoomMessageEvent(this.roomUuid, this.messages);
}

class PropertyEvent {
  final String roomUuid;
  final String? userUuid;
  final Map<String?, String?> properties;
  PropertyEvent(this.roomUuid, this.userUuid, this.properties);
}

class MemberRoleChangeEvent {
  final String roomUuid;
  final String userUuid;
  final RoomRole? role;
  MemberRoleChangeEvent(this.roomUuid, this.userUuid, this.role);
}

class MembersEvent {
  final String roomUuid;
  final List<RoomMember?>? members;
  final List<String?>? uuids;
  MembersEvent(this.roomUuid, this.members, this.uuids);
}

class CommonEvent {
  final String roomUuid;
  final String? userUuid;
  final int? arg1;
  final int? arg2;
  final String? sArg1;
  final String? sArg2;
  CommonEvent(this.roomUuid, this.userUuid, this.arg1, this.arg2, this.sArg1,
      this.sArg2);
}

class MemberNameChangeEvent {
  final String roomUuid;
  final String member;
  final String name;
  final RoomMember? operateBy;
  MemberNameChangeEvent(this.roomUuid, this.member, this.name, this.operateBy);
}

class MemberToggleEvent {
  final String roomUuid;
  final String userUuid;
  final String? operateUuid;
  final bool isOn;
  MemberToggleEvent(this.roomUuid, this.userUuid, this.operateUuid, this.isOn);
}

class MemberAudioConnectStateChangedEvent {
  final String roomUuid;
  final String userUuid;
  final bool isAudioConnected;
  MemberAudioConnectStateChangedEvent(
      this.roomUuid, this.userUuid, this.isAudioConnected);
}

class RoomToggleEvent {
  final String roomUuid;
  final bool isOn;
  RoomToggleEvent(this.roomUuid, this.isOn);
}

class MemberVolumeInfo {
  final String userUuid;
  final int volume;
  MemberVolumeInfo(this.userUuid, this.volume);
}

class LocalVolumeEvent {
  final String roomUuid;
  final int volume;
  final bool enableVad;
  LocalVolumeEvent(this.roomUuid, this.volume, this.enableVad);
}

class RtcVolumeEvent {
  final String roomUuid;
  final List<MemberVolumeInfo?>? volumes;
  final int totalVolume;

  RtcVolumeEvent(
    this.roomUuid,
    this.volumes,
    this.totalVolume,
  );
}

class WhiteboardShowFileChooserTypeEvent {
  final String roomUuid;
  final List<String?> types;

  WhiteboardShowFileChooserTypeEvent(
    this.roomUuid,
    this.types,
  );
}

class RtcVirtualBackgroundSourceEvent {
  final String roomUuid;
  final bool enabled;
  final int reason;

  RtcVirtualBackgroundSourceEvent(
    this.roomUuid,
    this.enabled,
    this.reason,
  );
}

class CreateAudioMixingOption {
  final String path;
  final int loopCount;
  final bool sendEnabled;
  final int sendVolume;
  final bool playbackEnabled;
  final int playbackVolume;
  final int startTimeStamp;
  final AudioStreamType? sendWithAudioType;
  CreateAudioMixingOption(
      {required this.path,
      required this.loopCount,
      required this.sendEnabled,
      required this.sendVolume,
      required this.playbackEnabled,
      required this.playbackVolume,
      required this.startTimeStamp,
      required this.sendWithAudioType});
}

class CreateAudioEffectOption {
  final String path;
  final int loopCount;
  final bool sendEnabled;
  final int sendVolume;
  final bool playbackEnabled;
  final int playbackVolume;
  final int startTimeStamp;
  final int progressInterval;
  final AudioStreamType? sendWithAudioType;
  CreateAudioEffectOption(
      {required this.path,
      required this.loopCount,
      required this.sendEnabled,
      required this.sendVolume,
      required this.playbackEnabled,
      required this.playbackVolume,
      required this.progressInterval,
      required this.startTimeStamp,
      required this.sendWithAudioType});
}

enum AudioStreamType {
  AudioStreamTypeMain,

  AudioStreamTypeSub
}

@FlutterApi()
abstract class PreviewRoomEventSink {
  void onRtcVirtualBackgroundSourceEnabled(
      RtcVirtualBackgroundSourceEvent event);
}

@FlutterApi()
abstract class RoomEventSink {
  void onRoomEnd(String roomUuid, String reason);

  void onRoomPropertiesChanged(PropertyEvent event);

  void onRoomPropertiesDeleted(PropertyEvent event);

  void onMemberPropertiesChanged(PropertyEvent event);

  void onMemberPropertiesDeleted(PropertyEvent event);

  void onMemberRoleChanged(MemberRoleChangeEvent event);

  void onMemberNameChanged(MemberNameChangeEvent event);

  void onMemberJoinRoom(MembersEvent event);

  void onMemberLeaveRoom(MembersEvent event);

  void onMemberJoinRtcChannel(MembersEvent event);

  void onMemberLeaveRtcChannel(MembersEvent event);

  void onRtcChannelError(CommonEvent event);

  void onMemberJoinChatroom(MembersEvent event);

  void onMemberLeaveChatroom(MembersEvent event);

  void onMemberAudioMuteChanged(MemberToggleEvent event);

  void onMemberVideoMuteChanged(MemberToggleEvent event);

  void onMemberScreenShareStateChanged(MemberToggleEvent event);

  void onReceiveChatroomMessages(RoomMessageEvent messages);

  void onChatroomMessageAttachmentProgress(CommonEvent messages);

  void onMemberWhiteboardShareStateChanged(MemberToggleEvent event);

  void onWhiteboardError(CommonEvent event);

  void onRoomLockStateChanged(RoomToggleEvent event);

  void onRoomLiveStateChanged(CommonEvent event);

  void onRtcAudioVolumeIndication(RtcVolumeEvent event);

  void onRtcAudioOutputDeviceChanged(CommonEvent event);

  void onLocalAudioVolumeIndicationwithVad(LocalVolumeEvent event);

  void onRtcVirtualBackgroundSourceEnabled(
      RtcVirtualBackgroundSourceEvent event);
  @async
  String? onWhiteboardShowFileChooser(WhiteboardShowFileChooserTypeEvent event);

  void onAudioMixingStateChanged(CommonEvent event);

  void onRoomRemainingSecondsRenewed(String roomUuid, int remainingSeconds);

  void onRoomConnectStateChanged(String roomUuid, int state);

  void onRoomCloudRecordStateChanged(
      String roomUuid, int state, String? operateBy);

  void onMemberAudioConnectStateChanged(
      MemberAudioConnectStateChangedEvent event);
}

@FlutterApi()
abstract class SeatEventSink {
  void onSeatManagerAdded(List<String> managers);

  void onSeatManagerRemoved(List<String> managers);

  void onSeatRequestSubmitted(int seatIndex, String user);

  void onSeatRequestCancelled(int seatIndex, String user);

  void onSeatRequestApproved(
      int seatIndex, String user, String operateBy, bool isAutoAgree);

  void onSeatRequestRejected(int seatIndex, String user, String operateBy);

  void onSeatInvitationReceived(int seatIndex, String user, String operateBy);

  void onSeatInvitationCancelled(int seatIndex, String user, String operateBy);

  void onSeatInvitationAccepted(int seatIndex, String user, bool isAutoAgree);

  void onSeatInvitationRejected(int seatIndex, String user);

  void onSeatLeave(int seatIndex, String user);

  void onSeatKicked(int seatIndex, String user, String operateBy);

  void onSeatListChanged(List<SeatItem> seatItems);
}

@HostApi(dartHostTestHandler: "RoomChatControllerApi")
abstract class RoomChatControllerApi {
  @async
  SimpleResponse joinChatroom(String roomUuid, int chatroomType);

  @async
  SimpleResponse leaveChatroom(String roomUuid, int chatroomType);

  @async
  ChatRoomMessageResponse sendBroadcastTextMessage(
      String roomUuid, String message, int chatroomType);

  @async
  ChatRoomMessageResponse sendDirectTextMessage(
      String roomUuid, String userUuid, String message, int chatroomType);

  @async
  ChatRoomMessageResponse sendGroupTextMessage(String roomUuid,
      List<String> userUuids, String message, int chatroomType);

  @async
  ChatRoomMembersResponse fetchChatRoomMembers(
      String roomUuid, int queryType, int limit, int chatroomType);

  @async
  ChatRoomMessagesResponse fetchChatroomHistoryMessages(
      String roomUuid, ChatroomHistoryMessageSearchOption option);

  @async
  SimpleResponse recallChatroomMessage(
      String roomUuid, String messageId, int messageTime, int chatroomType);

  @async
  ChatRoomMessageResponse sendImageMessage(
    String roomUuid,
    String messageUuid,
    String imagePath,
    int chatroomType,
    List<String> userUuids,
  );

  @async
  ChatRoomMessageResponse sendFileMessage(
    String roomUuid,
    String messageUuid,
    String filePath,
    int chatroomType,
    List<String> userUuids,
  );

  @async
  SimpleResponse downloadAttachment(
    String roomUuid,
    String messageUuid,
  );

  @async
  SimpleResponse cancelDownloadAttachment(
    String roomUuid,
    String messageUuid,
  );
}

@HostApi(dartHostTestHandler: "RoomSeatControllerApi")
abstract class RoomSeatControllerApi {
  @async
  SimpleResponse submitSeatRequest(
      String roomUuid, int seatIndex, bool exclusive);

  @async
  SimpleResponse cancelSeatRequest(String roomUuid);

  @async
  SimpleResponse leaveSeat(String roomUuid);

  @async
  SimpleResponse approveSeatRequest(String roomUuid, String user);

  @async
  SimpleResponse rejectSeatRequest(String roomUuid, String user);

  @async
  SimpleResponse kickSeat(String roomUuid, String user);

  @async
  SimpleResponse sendSeatInvitation(
      String roomUuid, int seatIndex, String user);

  @async
  SimpleResponse cancelSeatInvitation(String roomUuid, String user);

  @async
  SimpleResponse acceptSeatInvitation(String roomUuid);

  @async
  SimpleResponse rejectSeatInvitation(String roomUuid);

  @async
  SimpleResponse closeSeats(String roomUuid, List<int> seatIndices);

  @async
  SimpleResponse openSeats(String roomUuid, List<int> seatIndices);

  @async
  SeatInfoResponse getSeatInfo(String roomUuid);

  @async
  SeatRequestListResponse getSeatRequestList(String roomUuid);

  @async
  SeatInvitationListResponse getSeatInvitationList(String roomUuid);
}

class RoomLiveStreamTaskInfo {
  final String taskID;
  final String streamURL;
  final bool serverRecordEnabled;
  final String? extraInfo;
  final RoomLiveStreamLayout? layout;
  final RoomLiveConfig? config;
  final int lsMode;

  RoomLiveStreamTaskInfo({
    required this.taskID,
    required this.streamURL,
    required this.serverRecordEnabled,
    required this.layout,
    this.extraInfo,
    required this.config,
    required this.lsMode,
  });
}

class RoomLiveStreamLayout {
  final int width;
  final int height;
  final int backgroundColor;
  final List<RoomLiveStreamUserTranscoding?>? users;
  final RoomLiveStreamImageInfo? bgImage;

  RoomLiveStreamLayout({
    required this.width,
    required this.height,
    required this.backgroundColor,
    required this.users,
    required this.bgImage,
  });
}

class RoomLiveStreamUserTranscoding {
  final String userUuid;
  final bool videoPush;
  final int x;
  final int y;
  final int width;
  final int height;
  final bool audioPush;
  final int zOrder;
  final int adaption;

  RoomLiveStreamUserTranscoding({
    required this.userUuid,
    required this.x,
    required this.y,
    required this.width,
    required this.height,
    required this.videoPush,
    required this.audioPush,
    required this.zOrder,
    required this.adaption,
  });
}

class RoomLiveStreamImageInfo {
  final String? url;
  final int x;
  final int y;
  final int width;
  final int height;

  RoomLiveStreamImageInfo({
    this.url,
    required this.x,
    required this.y,
    required this.width,
    required this.height,
  });
}

class RoomLiveConfig {
  final bool singleVideoPassthrough;
  final int audioBitrate;
  final int sampleRate;
  final int channels;
  final int audioCodecProfile;

  RoomLiveConfig({
    required this.singleVideoPassthrough,
    required this.audioBitrate,
    required this.sampleRate,
    required this.channels,
    required this.audioCodecProfile,
  });
}

@HostApi(dartHostTestHandler: "RoomRtcControllerApi")
abstract class RoomRtcControllerApi {
  @async
  SimpleResponse startPreview(String roomUuid);

  @async
  SimpleResponse stopPreview(String roomUuid, bool releaseRtc);

  @async
  SimpleResponse joinRtcChannel(String roomUuid);

  @async
  SimpleResponse leaveRtcChannel(String roomUuid);

  @async
  SimpleResponse muteMyAudio(String roomUuid, bool enableMediaPub);

  @async
  SimpleResponse unmuteMyAudio(String roomUuid, bool enableMediaPub);

  @async
  SimpleResponse enableMediaPub(String roomUuid, int mediaType, bool enable);

  @async
  SimpleResponse pauseLocalAudioRecording(String roomUuid);

  @async
  SimpleResponse resumeLocalAudioRecording(String roomUuid);

  @async
  SimpleResponse muteMemberAudio(String roomUuid, String uuid);

  @async
  SimpleResponse unmuteMemberAudio(String roomUuid, String uuid);

  @async
  SimpleResponse muteMemberVideo(String roomUuid, String uuid);

  @async
  SimpleResponse unmuteMemberVideo(String roomUuid, String uuid);

  @async
  SimpleResponse pauseLocalVideoCapture(String roomUuid);

  @async
  SimpleResponse resumeLocalVideoCapture(String roomUuid);

  @async
  SimpleResponse unmuteMyVideo(String roomUuid);

  @async
  SimpleResponse muteMyVideo(String roomUuid);

  @async
  SimpleResponse stopScreenShare(String roomUuid);

  @async
  SimpleResponse startScreenShare(StartScreenShareRequest request);

  @async
  SimpleResponse stopMemberScreenShare(String roomUuid, String uuid);

  @async
  SimpleResponse subscribeRemoteVideoStream(
      String roomUuid, String userUuid, int streamType, bool subscribe);

  @async
  SimpleResponse subscribeRemoteAudioStream(
      String roomUuid, String userUuid, bool subscribe);

  @async
  SimpleResponse subscribeRemoteSubStreamVideo(
      String roomUuid, String userUuid, bool subscribe);

  @async
  SimpleResponse setSpeakerphoneOn(String roomUuid, bool on);

  @async
  SimpleResponse isSpeakerphoneOn(
    String roomUuid,
  );

  @async
  SimpleResponse stopAudioDump(
    String roomUuid,
  );

  @async
  SimpleResponse startAudioDump(String roomUuid, int type);

  @async
  SimpleResponse enableAudioVolumeIndication(
      String roomUuid, bool enable, int interval, bool vad);

  @async
  SimpleResponse startRtcChannelMediaRelay(String roomUuid);

  @async
  SimpleResponse stopRtcChannelMediaRelay(String roomUuid);

  @async
  SimpleResponse enableEarBack(String roomUuid, bool enable, int volume);

  @async
  SimpleResponse startAudioMixing(
      String roomUuid, CreateAudioMixingOption option);

  @async
  SimpleResponse playEffect(
      String roomUuid, int id, CreateAudioEffectOption option);

  @async
  SimpleResponse stopAudioMixing(String roomUuid);

  @async
  SimpleResponse stopEffect(String roomUuid, int id);

  @async
  SimpleResponse setAudioMixingSendVolume(String roomUuid, int volume);

  @async
  SimpleResponse setAudioMixingPlaybackVolume(String roomUuid, int volume);

  @async
  SimpleResponse setEffectSendVolume(String roomUuid, int id, int volume);

  @async
  SimpleResponse setEffectPlaybackVolume(String roomUuid, int id, int volume);

  @async
  SimpleResponse stopAllEffects(String roomUuid);

  @async
  SimpleResponse adjustUserPlaybackSignalVolume(
      String roomUuid, String userUuid, int volume);

  @async
  SimpleResponse adjustPlaybackSignalVolume(String roomUuid, int volume);

  @async
  SimpleResponse switchCamera(String roomUuid);

  @async
  SimpleResponse addBeautyFilter(String roomUuid, String path);

  @async
  SimpleResponse addBeautySticker(String roomUuid, String path);

  @async
  SimpleResponse enableBeauty(String roomUuid, bool isOpenBeauty);

  @async
  SimpleResponse removeBeautyFilter(String roomUuid);

  @async
  SimpleResponse removeBeautySticker(String roomUuid);

  @async
  SimpleResponse startBeauty(String roomUuid);

  @async
  SimpleResponse stopBeauty(String roomUuid);

  @async
  SimpleResponse setBeautyFilterLevel(String roomUuid, double level);

  @async
  SimpleResponse setBeautyEffect(String roomUuid, int beautyType, double level);

  @async
  SimpleResponse enableVirtualBackground(String roomUuid, bool enabled,
      NERoomVirtualBackgroundSource backgroundSource);

  @async
  SimpleResponse pauseAudioMixing(String roomUuid);

  @async
  SimpleResponse adjustRecordingSignalVolume(String roomUuid, int volume);

  @async
  SimpleResponse resumeAudioMixing(String roomUuid);

  @async
  SimpleResponse setRecordDeviceMute(String roomUuid, bool mute);

  @async
  SimpleResponse setClientRole(String roomUuid, int role);

  @async
  SimpleResponse setLocalAudioProfile(
      String roomUuid, int profile, int scenario);

  @async
  SimpleResponse setChannelProfile(String roomUuid, int value);

  @async
  SimpleResponse enableAudioAINS(String roomUuid, bool enable);

  @async
  SimpleResponse enableLoopbackRecording(String roomUuid, bool enable);

  @async
  SimpleResponse enableEncryption(
      String roomUuid, String gmEncryptKey, int encryptionMode);

  @async
  SimpleResponse disableEncryption(String roomUuid);

  @async
  SimpleResponse setParameters(String roomUuid, Map<String, Object> params);

  @async
  SimpleResponse disconnectMyAudio(String roomUuid);

  @async
  SimpleResponse reconnectMyAudio(String roomUuid);

  // @async
  // SimpleResponse adjustLoopbackRecordingSignalVolume(
  //     String roomUuid, int volume);
}

@HostApi(dartHostTestHandler: "PreviewRoomRtcControllerApi")
abstract class PreviewRoomRtcControllerApi {
  @async
  SimpleResponse startPreview();

  @async
  SimpleResponse stopPreview(bool releaseRtc);

  @async
  SimpleResponse switchCamera();

  @async
  SimpleResponse addBeautyFilter(String path);

  @async
  SimpleResponse addBeautySticker(String path);

  @async
  SimpleResponse enableBeauty(bool isOpenBeauty);

  @async
  SimpleResponse removeBeautyFilter();

  @async
  SimpleResponse removeBeautySticker();

  @async
  SimpleResponse startBeauty();

  @async
  SimpleResponse stopBeauty();

  @async
  SimpleResponse setBeautyFilterLevel(double level);

  @async
  SimpleResponse setBeautyEffect(int beautyType, double level);

  @async
  SimpleResponse enableVirtualBackground(
      bool enabled, NERoomVirtualBackgroundSource backgroundSource);
}

class NERoomVirtualBackgroundSource {
  int backgroundSourceType;
  int color;
  String source;

  int blurDegree;

  NERoomVirtualBackgroundSource({
    required this.backgroundSourceType,
    required this.color,
    required this.source,
    required this.blurDegree,
  });
}

class StartScreenShareRequest {
  final String roomUuid;
  final int? videoProfile;
  final int? frameRate;
  final String? iosAppGroup;

  StartScreenShareRequest(
      {required this.roomUuid,
      this.videoProfile,
      this.frameRate,
      this.iosAppGroup});
}

class VideoRendererMirrorRequest {
  final int textureId;
  final bool mirror;
  final String roomUuid;

  VideoRendererMirrorRequest(this.textureId, this.mirror, this.roomUuid);
}

class TextureIdRendererRequest {
  final String roomUuid;
  final String? userUuid;
  final int textureId;

  TextureIdRendererRequest(this.roomUuid, this.userUuid, this.textureId);
}

@HostApi(dartHostTestHandler: "RoomRtcRenderApi")
abstract class RoomRtcVideoRender {
  @async
  SimpleResponse createVideoRenderer(String roomUuid);

  @async
  SimpleResponse setupLocalVideoRenderer(int textureId);

  // Future<VoidResult> setupLocalSubStreamVideoRenderer(
  //     TextureIdRendererRequest textureIdRendererRequest);

  @async
  SimpleResponse detachLocalVideoRenderer(bool subStream, int textureId);

  @async
  SimpleResponse detachRemoteVideoRenderer(
      String userUuid, bool subStream, int textureId);

  @async
  SimpleResponse setMirror(int textureId, bool mirror);

  @async
  SimpleResponse setupRemoteSubStreamVideoRenderer(
      String userUuid, int textureId);

  @async
  SimpleResponse setupRemoteVideoRenderer(String userUuid, int textureId);

  @async
  SimpleResponse disposeVideoRenderer(int textureId);
}

@HostApi(dartHostTestHandler: "RoomWhiteboardControllerApi")
abstract class RoomWhiteboardControllerApi {
  @async
  SimpleResponse startWhiteboardShare(String roomUuid);

  @async
  SimpleResponse stopWhiteboardShare(String roomUuid);

  @async
  SimpleResponse setEnableDraw(String roomUuid, bool enable);

  @async
  SimpleResponse stopMemberWhiteboardShare(String roomUuid, String userUuid);

  @async
  SimpleResponse setCanvasBackgroundColor(String roomUuid, String color);

  @async
  SimpleResponse lockCameraWithContent(String roomUuid, int width, int height);
}

@FlutterApi()
abstract class MessageEventSink {
  void onReceivePassThroughMessage(PassThroughMessageEvent messageEvent);
}

class PassThroughMessageEvent {
  final String? roomUuid;
  final String? senderUuid;
  final String data;
  final int commandId;
  PassThroughMessageEvent(
      this.roomUuid, this.senderUuid, this.data, this.commandId);
}

@HostApi()
abstract class MessageServiceApi {
  @async
  SimpleResponse sendCustomMessage(String? roomUuid, String userUuid,
      int commandId, String data, InjectedAuthorization? injectedAuthorization);
}

class RoomLiveRequest {
  final String title;
  final int liveLayout;
  final String? password;
  final List<String?> userUuidList;
  String? extensionConfig;

  RoomLiveRequest(this.title, this.liveLayout, this.password, this.userUuidList,
      this.extensionConfig);
}

class RoomLiveInfo {
  final String? taskId;
  final String? pushUrl;
  final String? chatRoomId;
  final String? httpPullUrl;
  final String? rtmpPullUrl;
  final String? hlsPullUrl;
  final String? title;
  final int? liveLayout;
  final String? password;
  final List<String?>? userUuidList;
  final int state;
  String? extensionConfig;

  RoomLiveInfo(
    this.title,
    this.userUuidList,
    this.password,
    this.liveLayout,
    this.chatRoomId,
    this.hlsPullUrl,
    this.httpPullUrl,
    this.pushUrl,
    this.rtmpPullUrl,
    this.taskId,
    this.state,
    this.extensionConfig,
  );
}

class RoomLiveResponse {
  final int code;

  final String? msg;

  final RoomLiveInfo? liveInfo;

  RoomLiveResponse(this.code, this.msg, this.liveInfo);
}

@HostApi(dartHostTestHandler: "RoomLiveControllerApi")
abstract class RoomLiveControllerApi {
  @async
  RoomLiveResponse startLive(String roomUuid, RoomLiveRequest request);

  @async
  SimpleResponse stopLive(String roomUuid);

  @async
  RoomLiveResponse updateLive(String roomUuid, RoomLiveRequest request);

  @async
  RoomLiveResponse getLiveInfo(String roomUuid);

  @async
  SimpleResponse addLiveStreamTask(
      String roomUuid, RoomLiveStreamTaskInfo taskInfo);

  @async
  SimpleResponse updateLiveStreamTask(
      String roomUuid, RoomLiveStreamTaskInfo taskInfo);

  @async
  SimpleResponse removeLiveStreamTask(String roomUuid, String taskID);
}

class ChatRoomMember {
  final String? nick;
  final String? avatar;

  ChatRoomMember(
    this.nick,
    this.avatar,
  );
}

class ChatRoomMembersResponse {
  final int code;

  final String? msg;

  final List<ChatRoomMember?> chatRoomMembers;

  ChatRoomMembersResponse(this.code, this.msg, this.chatRoomMembers);
}

class ChatroomHistoryMessageSearchOption {
  final int limit;
  final int startTime;
  final int order;
  final int chatroomType;
  ChatroomHistoryMessageSearchOption(
      this.limit, this.startTime, this.order, this.chatroomType);
}

class ChatRoomMessagesResponse {
  final int code;

  final String? msg;

  final List<RoomMessage?> messages;

  ChatRoomMessagesResponse(this.code, this.msg, this.messages);
}

class ChatRoomMessageResponse {
  final int code;
  final String? msg;
  final RoomMessage? message;
  ChatRoomMessageResponse(this.code, this.msg, this.message);
}

class SeatInfoResponse {
  final int code;
  final String? msg;
  final SeatInfo? seatInfo;

  SeatInfoResponse(this.code, this.msg, this.seatInfo);
}

class SeatInfo {
  final String? creator;
  final List<String?>? managers;
  final List<SeatItem?>? seatItems;
  SeatInfo(this.creator, this.managers, this.seatItems);
}

class SeatItem {
  final int index;
  final int status;
  final String? user;
  final String? userName;
  final String? icon;
  final int onSeatType;
  final int updated;

  SeatItem(this.index, this.status, this.user, this.userName, this.icon,
      this.onSeatType, this.updated);
}

class SeatRequestListResponse {
  final int code;

  final String? msg;

  final List<SeatRequestItem?> seatRequestList;

  SeatRequestListResponse(this.code, this.msg, this.seatRequestList);
}

class SeatRequestItem {
  final int index;
  final String user;
  final String? userName;
  final String? icon;
  SeatRequestItem(this.index, this.user, this.userName, this.icon);
}

class SeatInvitationListResponse {
  final int code;

  final String? msg;

  final List<SeatInvitationItem?> seatInvitationList;

  SeatInvitationListResponse(this.code, this.msg, this.seatInvitationList);
}

class SeatInvitationItem {
  final int index;
  final String user;
  SeatInvitationItem(this.index, this.user);
}

class RoomStatsEvent {
  final int txBytes;
  final int rxBytes;
  final int cpuAppUsage;
  final int cpuTotalUsage;
  final int memoryAppUsageRatio;
  final int memoryTotalUsageRatio;
  final int memoryAppUsageInKBytes;
  final int totalDuration;
  final int txAudioBytes;
  final int txVideoBytes;
  final int rxAudioBytes;
  final int rxVideoBytes;
  final int rxAudioKBitRate;
  final int rxVideoKBitRate;
  final int txAudioKBitRate;
  final int txVideoKBitRate;
  final int upRtt;
  final int downRtt;
  final int txAudioPacketLossRate;
  final int txVideoPacketLossRate;
  final int txAudioPacketLossSum;
  final int txVideoPacketLossSum;
  final int txAudioJitter;
  final int txVideoJitter;
  final int rxAudioPacketLossRate;
  final int rxVideoPacketLossRate;
  final int rxAudioPacketLossSum;
  final int rxVideoPacketLossSum;
  final int rxAudioJitter;
  final int rxVideoJitter;

  RoomStatsEvent(
      this.txBytes,
      this.rxBytes,
      this.cpuAppUsage,
      this.cpuTotalUsage,
      this.memoryAppUsageRatio,
      this.memoryTotalUsageRatio,
      this.memoryAppUsageInKBytes,
      this.totalDuration,
      this.txAudioBytes,
      this.txVideoBytes,
      this.rxAudioBytes,
      this.rxVideoBytes,
      this.rxAudioKBitRate,
      this.rxVideoKBitRate,
      this.txAudioKBitRate,
      this.txVideoKBitRate,
      this.upRtt,
      this.downRtt,
      this.txAudioPacketLossRate,
      this.txVideoPacketLossRate,
      this.txAudioPacketLossSum,
      this.txVideoPacketLossSum,
      this.txAudioJitter,
      this.txVideoJitter,
      this.rxAudioPacketLossRate,
      this.rxVideoPacketLossRate,
      this.rxAudioPacketLossSum,
      this.rxVideoPacketLossSum,
      this.rxAudioJitter,
      this.rxVideoJitter);
}

class RoomNetworkQuality {
  final String? userId;
  final int upStatus;
  final int downStatus;

  RoomNetworkQuality(this.userId, this.upStatus, this.downStatus);
}

@FlutterApi()
abstract class RoomStatsSink {
  void onRtcStats(String roomUuid, RoomStatsEvent event);
  void onNetworkQuality(String roomUuid, List<RoomNetworkQuality> qualities);
}

class WaitingRoomMember {
  final String uuid;
  final String name;
  final String? avatar;
  final int joinTime;
  final int status;

  WaitingRoomMember({
    required this.uuid,
    required this.name,
    required this.avatar,
    required this.joinTime,
    required this.status,
  });
}

class WaitingRoomInfo {
  final int memberCount;
  final bool isEnabledOnEntry;
  final String? backgroundImageUrl;

  WaitingRoomInfo({
    required this.memberCount,
    required this.isEnabledOnEntry,
    this.backgroundImageUrl,
  });
}

@FlutterApi()
abstract class WaitingRoomEventSink {
  void onMyWaitingRoomStatusChanged(String roomUuid, int status, int reason);
  void onWaitingRoomInfoUpdated(String roomUuid, WaitingRoomInfo info);
  void onMemberJoin(String roomUuid, WaitingRoomMember member, int reason);
  void onMemberLeave(String roomUuid, String member, int reason);
  void onMemberAdmitted(String roomUuid, String member);
  void onMemberNameChanged(
      String roomUuid, String member, String name, RoomMember? operateBy);
}

class WaitingRoomGetMemberListResult {
  final int code;
  final String? msg;
  final List<WaitingRoomMember?>? members;

  WaitingRoomGetMemberListResult(this.code, this.msg, this.members);
}

@HostApi(dartHostTestHandler: "RoomWaitingRoomControllerApi")
abstract class WaitingRoomControllerApi {
  @async
  SimpleResponse admitMember(String roomUuid, String userUuid);
  @async
  SimpleResponse expelMember(
    String roomUuid,
    String userUuid,
    bool disallowRejoin,
  );
  @async
  SimpleResponse putInWaitingRoom(String roomUuid, String userUuid);
  @async
  SimpleResponse enableWaitingRoomOnEntry(String roomUuid);
  @async
  SimpleResponse disableWaitingRoomOnEntry(String roomUuid, bool admitAll);
  @async
  WaitingRoomGetMemberListResult getMemberList(
      String roomUuid, int joinTime, int size, bool asc);
  @async
  SimpleResponse changeMemberName(
      String roomUuid, String userUuid, String name);
}
